-- Moves the balls to random postions, rotates them

local lg = love.graphics
local random = math.random
local deg2rad = math.pi / 180


ballTest = {}

-- This is called in love.load
function ballTest.load()
	ballTest.name = "Basketball Test"						-- Name of the test
	ballTest.sprite = lg.newImage("data/basketball.png")	-- Sprite to load
	ballTest.samples = 700									-- Start amount

	-- Fill table with entities
	ballTest.entities = {}
	for i = 1, #ballTest do ballTest.addEntity() end

	-- Other data
	ballTest.spriteWidth = ballTest.sprite:getWidth()
	ballTest.spriteHeight = ballTest.sprite:getHeight()
end

-- This is called in love.update
function ballTest.update(dt)
	-- Add new entities if necessary
	ballTest.samples = synchronize(ballTest.samples)

	if ballTest.samples > #ballTest.entities then
		for i = 1, ballTest.samples - #ballTest.entities + 1 do ballTest.addEntity() end
	end

	-- Move entities
	for i = 1, ballTest.samples do
		local e = ballTest.entities[i]

		-- Move and rotate
		e.x = e.x + e.dx * e.v * dt
		e.y = e.y + e.dy * e.v * dt

		e.r = e.r + e.t * dt
		e.r = e.r < 0 and e.r + 360 or e.r
		e.r = e.r > 360 and e.r - 360 or e.r
		
		-- Bounce off the edges
		local wp2 = e.w / 2
		local hp2 = e.h / 2

		if e.x < wp2 then
			e.x = wp2
			e.dx = -e.dx
		elseif e.x > screenWidth - wp2 then
			e.x = screenWidth - wp2
			e.dx = -e.dx
		end

		if e.y < hp2 then
			e.y = hp2
			e.dy = -e.dy
		elseif e.y > screenHeight - hp2 then
			e.y = screenHeight - hp2
			e.dy = -e.dy
		end
	end
end

-- This is called in love.draw
function ballTest.draw()
	lg.setColor(255, 255, 255, 255)

    for i = 1, ballTest.samples do
		local e = ballTest.entities[i]
		lg.draw(ballTest.sprite, e.x, e.y, e.r * deg2rad, e.s, e.s, ballTest.spriteWidth / 2, ballTest.spriteHeight / 2)
    end
end

-- This is also called in love.draw
function ballTest.debug()
	printDebug("Balls", ballTest.samples)
end

-- Adds a new sprite to the table
function ballTest.addEntity()
	local e = {}

    e.s = 0.1 + random(5) * 0.03
	e.w = ballTest.spriteWidth * e.s
	e.h = ballTest.spriteHeight * e.s
    e.x = (e.w / 2) + random(screenWidth - e.w)
    e.y = (e.h / 2) + random(screenHeight - e.h)
	e.r = random(360)
	e.t = (-10 + random(20)) * 10
	e.dx = random(2) == 1 and random(5) or -random(5)
	e.dy = random(2) == 1 and random(5) or -random(5)
	e.v = 10 + random(10) * 2
	
	table.insert(ballTest.entities, e) 
end
