-- Moves the bugs to random postions, rotates them

local lg = love.graphics
local random = math.random
local deg2rad = math.pi / 180
local sin = math.sin
local cos = math.cos


bugTest = {}

-- This is called in love.load
function bugTest.load()
	bugTest.name = "Ladybug Test"						-- Name of the test
	bugTest.sprite = lg.newImage("data/ladybug.png")	-- Sprite to load
	bugTest.samples = 800								-- Start amount

	-- Fill table with entities
	bugTest.entities = {}
	for i = 1, #bugTest do bugTest.addEntity() end

	-- Other data
	bugTest.spriteWidth = bugTest.sprite:getWidth()
	bugTest.spriteHeight = bugTest.sprite:getHeight()
end

-- This is called in love.update
function bugTest.update(dt)
	-- Add new entities if necessary
	bugTest.samples = synchronize(bugTest.samples)

	if bugTest.samples > #bugTest.entities then
		for i = 1, bugTest.samples - #bugTest.entities + 1 do bugTest.addEntity() end
	end

	-- Move entities
	for i = 1, bugTest.samples do
		local e = bugTest.entities[i]

		-- Rotate to target direction
		local p = 0

		if e.t > e.r then
			p = e.t - e.r
		else
			p = e.t + (360 - e.r)
		end

		if p < 180 then
			e.r = e.r + 100 * dt
		else
			e.r = e.r - 100 * dt
		end

		e.r = bugTest.trimRotation(e.r)

		-- Move in direction
		e.x = e.x + cos(e.r * deg2rad) * e.v * dt
		e.y = e.y + sin(e.r * deg2rad) * e.v * dt
		
		-- If reached the edges
		local wp2 = e.w / 2
		local hp2 = e.h / 2

		if e.x < wp2 then
			e.x = wp2
		elseif e.x > screenWidth - wp2 then
			e.x = screenWidth - wp2
		end

		if e.y < hp2 then
			e.y = hp2
		elseif e.y > screenHeight - hp2 then
			e.y = screenHeight - hp2
		end

		-- Chnage direction?
		e.c = e.c - dt
		if e.c < 0 then
			e.t = random(360)
			e.c = random(5)
		end
	end
end

-- This is called in love.draw
function bugTest.draw()
	lg.setColor(255, 255, 255, 255)

    for i = 1, bugTest.samples do
		local e = bugTest.entities[i]
		lg.draw(bugTest.sprite, e.x, e.y, e.r * deg2rad, e.s, e.s, bugTest.spriteWidth / 2, bugTest.spriteHeight / 2)
    end
end

-- This is also called in love.draw
function bugTest.debug()
	printDebug("Bugs", bugTest.samples)
end

-- Adds a new sprite to the table
function bugTest.addEntity()
	local e = {}

    e.s = 0.1 + random(5) * 0.03
	e.w = bugTest.spriteWidth * e.s
	e.h = bugTest.spriteHeight * e.s
    e.x = (e.w / 2) + random(screenWidth - e.w)
    e.y = (e.h / 2) + random(screenHeight - e.h)
	e.r = random(360)
	e.t = random(360)
	e.v = 10 + random(10) * 5
	e.c = random(5)
	
	table.insert(bugTest.entities, e) 
end

-- Trims the angles if bigger than 360 or smaller than 0
function bugTest.trimRotation(rotation)
	local r = rotation
	r = r < 0 and r + 360 or r
	r = r > 360 and r - 360 or r
	return r
end
